"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = require("assert");
// eslint-disable-next-line import/no-internal-modules
const duplicateBatchDetector_js_1 = require("../../opLifecycle/duplicateBatchDetector.js");
/**
 * Helper function to create (enough of) a BatchStartInfo for testing.
 * Inbound batch may have explicit batchId, or merely clientId and batchStartCsn and batchId must be computed - allow either as inputs
 */
function makeBatch({ sequenceNumber, minimumSequenceNumber, batchId, clientId, batchStartCsn, }) {
    return {
        keyMessage: {
            sequenceNumber,
            minimumSequenceNumber,
        },
        batchId,
        clientId,
        batchStartCsn,
    };
}
describe("DuplicateBatchDetector", () => {
    // expose private members for testing
    let detector;
    let seqNum;
    beforeEach("setup", () => {
        seqNum = 1;
        detector = new duplicateBatchDetector_js_1.DuplicateBatchDetector();
    });
    afterEach("validation", () => {
        assert_1.strict.deepEqual([...detector.batchIdsAll].sort(), [...detector.batchIdsBySeqNum].map(([, batchId]) => batchId).sort(), "Invariant: batchIdsAll and batchIdsBySeqNum should be in sync");
    });
    it("First inbound batch is not a duplicate", () => {
        const inboundBatch = makeBatch({
            sequenceNumber: seqNum++,
            minimumSequenceNumber: 0,
            batchId: "batch1",
        });
        const result = detector.processInboundBatch(inboundBatch);
        assert_1.strict.deepEqual(result, { duplicate: false });
    });
    it("Different inbound batches are not duplicates", () => {
        const inboundBatch1 = makeBatch({
            sequenceNumber: seqNum++,
            minimumSequenceNumber: 0,
            batchId: "batch1",
        });
        const inboundBatch2 = makeBatch({
            sequenceNumber: seqNum++,
            minimumSequenceNumber: 0,
            batchId: "batch2",
        });
        detector.processInboundBatch(inboundBatch1);
        const result = detector.processInboundBatch(inboundBatch2);
        assert_1.strict.deepEqual(result, { duplicate: false });
    });
    it("Matching inbound batches are duplicates", () => {
        const inboundBatch1 = makeBatch({
            sequenceNumber: seqNum++, // 1
            minimumSequenceNumber: 0,
            batchId: "batch1",
        });
        const inboundBatch2 = makeBatch({
            sequenceNumber: seqNum++, // 2
            minimumSequenceNumber: 0,
            batchId: "batch1",
        });
        detector.processInboundBatch(inboundBatch1);
        const result = detector.processInboundBatch(inboundBatch2);
        assert_1.strict.deepEqual(result, { duplicate: true, otherSequenceNumber: 1 });
    });
    it("Matching inbound batches, one with batchId one without, are duplicates", () => {
        const inboundBatch1 = makeBatch({
            sequenceNumber: seqNum++, // 1
            minimumSequenceNumber: 0,
            batchId: "clientId_[33]",
        });
        const inboundBatch2 = makeBatch({
            sequenceNumber: seqNum++, // 2
            minimumSequenceNumber: 0,
            clientId: "clientId",
            batchStartCsn: 33,
        });
        detector.processInboundBatch(inboundBatch1);
        const result = detector.processInboundBatch(inboundBatch2);
        assert_1.strict.deepEqual(result, { duplicate: true, otherSequenceNumber: 1 });
    });
    it("should clear old batchIds that are no longer at risk for duplicates", () => {
        const inboundBatch1 = makeBatch({
            sequenceNumber: seqNum++, // 1
            minimumSequenceNumber: 0,
            batchId: "batch1",
        });
        const inboundBatch2 = makeBatch({
            sequenceNumber: seqNum++, // 2
            minimumSequenceNumber: 0,
            batchId: "batch2",
        });
        const inboundBatch3 = makeBatch({
            sequenceNumber: seqNum++, // 3
            minimumSequenceNumber: 2,
            batchId: "batch3",
        });
        detector.processInboundBatch(inboundBatch1);
        detector.processInboundBatch(inboundBatch2);
        assert_1.strict.deepEqual([...detector.batchIdsAll].sort(), ["batch1", "batch2"], "Incorrect batchIds (after 2)");
        detector.processInboundBatch(inboundBatch3);
        assert_1.strict.deepEqual([...detector.batchIdsAll].sort(), ["batch2", "batch3"], "Incorrect batchIds (after 3)");
    });
});
//# sourceMappingURL=duplicateBatchDetector.spec.js.map